<?php
/**
 * Copyright (C) 2017-2019 Petr Hucik <petr@getdatakick.com>
 *
 * NOTICE OF LICENSE
 *
 * Licensed under the DataKick Regular License version 1.0
 * For more information see LICENSE.txt file
 *
 * @author    Petr Hucik <petr@getdatakick.com>
 * @copyright 2017-2019 Petr Hucik
 * @license   Licensed under the DataKick Regular License version 1.0
 */

namespace Conseqs\Actions;

use Combination;
use Conseqs\Action;
use Conseqs\ParameterDefinitions;
use Conseqs\Parameters\IntParameterDefinition;
use Conseqs\ParameterValues;
use Conseqs\RuntimeModifier;
use Context;
use Db;
use DbQuery;
use Order;
use OrderDetail;
use PrestaShopException;
use Product;
use StockAvailable;
use Validate;

class AddFreeProductToOrder extends Action
{
    /**
     * @return string
     */
    public function getName()
    {
        return $this->l('Add free product to order');
    }

    /**
     * @return ParameterDefinitions
     */
    public function getSettingsParameters()
    {
        return ParameterDefinitions::none();
    }


    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->l('This action will add fee product to existing order. This does not have impact on total order price');
    }

    /**
     * @param ParameterValues $settings
     * @return ParameterDefinitions
     */
    public function getInputParameters(ParameterValues $settings)
    {
        return new ParameterDefinitions([
            'params.orderId' => new IntParameterDefinition($this->l('Order ID')),
            'params.productId' => new IntParameterDefinition($this->l('Product ID')),
            'params.combinationId' => new IntParameterDefinition($this->l('Combination ID')),
            'params.quantity' => new IntParameterDefinition($this->l('Quantity')),
        ]);
    }

    /**
     * @param ParameterValues $settings
     * @param ParameterValues $input
     * @param ParameterValues $triggerOutput
     * @param RuntimeModifier $runtimeModifier
     * @return bool
     * @throws PrestaShopException
     */
    public function execute(ParameterValues $settings, ParameterValues $input, ParameterValues $triggerOutput, RuntimeModifier $runtimeModifier)
    {
        $orderId = (int)$input->getValue('params.orderId');
        $productId = (int)$input->getValue('params.productId');
        $combinationId = (int)$input->getValue('params.combinationId');
        $quantity = (int)$input->getValue('params.quantity');

        if ($quantity < 1) {
            throw new PrestaShopException("Quantity must be higher than zero");
        }

        $langId = Context::getContext()->language->id;
        $order = new Order($orderId, $langId);
        if (! Validate::isLoadedObject($order)) {
            throw new PrestaShopException("Order with id $orderId not found");
        }

        $product = new Product($productId, true, $langId);
        if (! Validate::isLoadedObject($product)) {
            throw new PrestaShopException("Product with id $productId not found");
        }

        $reference = $product->name;
        $ean13 = $product->ean13;
        $upc = $product->upc;
        $weight = $product->weight;

        if ($combinationId) {
            $combination = new Combination($combinationId, $langId);
            if (!Validate::isLoadedObject($combination)) {
                throw new PrestaShopException("Combination with id $combinationId not found");
            }
            if ((int)$combination->id_product !== $productId) {
                throw new PrestaShopException("Combination with id $combinationId does not belong to product with id $productId");
            }
            if ($combination->reference) {
                $reference = $combination->reference;
            }
            if ($combination->upc) {
                $upc = $combination->upc;
            }
            if ($combination->ean13) {
                $upc = $combination->ean13;
            }
            if ($combination->weight) {
                $weight = $combination->weight;
            }
        }

        $quantityAvailable = StockAvailable::getQuantityAvailableByProduct($productId, $combinationId);
        if (($quantityAvailable < $quantity) && (!$product->isAvailableWhenOutOfStock((int) $product->out_of_stock))) {
            $id = $productId;
            if ($combinationId) {
                $id .= '/combination '.$combinationId;
            }
            throw new PrestaShopException("Product $id is not in stock");
        }

        $orderDetail = new OrderDetail();
        $orderDetail->id_shop = $order->id_shop;
        $orderDetail->id_order = $orderId;
        $orderDetail->product_id = $productId;
        $orderDetail->product_attribute_id = $combinationId;
        $orderDetail->product_quantity = $quantity;
        $orderDetail->product_quantity_in_stock = $quantityAvailable;
        $orderDetail->product_price = 0.0;
        $orderDetail->product_name = Product::getProductName($productId, $combinationId, $langId);
        $orderDetail->id_warehouse = $this->getWarehouse($orderId);
        $orderDetail->id_order_invoice = $this->getOrderInvoice($orderId);
        $orderDetail->product_ean13 = $ean13;
        $orderDetail->product_upc = $upc;
        $orderDetail->product_reference = $reference;
        $orderDetail->product_weight = $weight;

        if ($orderDetail->add()) {
            return StockAvailable::updateQuantity($productId, $combinationId, -$quantity);
        }
        return false;
    }

    /**
     * @param int $orderId
     *
     * @return int
     * @throws PrestaShopException
     */
    private function getWarehouse($orderId)
    {
        return (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue(
            (new DbQuery())
                ->select('`id_order_invoice`')
                ->from('order_invoice')
                ->where('`id_order` = '.(int)$orderId)
        );
    }

    /**
     * @param int $orderId
     *
     * @return int
     * @throws PrestaShopException
     */
    private function getOrderInvoice($orderId)
    {
        return (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue(
            (new DbQuery())
                ->select('max(id_warehouse)')
                ->from('order_detail')
                ->where('id_order = '.(int) $orderId)
        );
    }
}