<?php
/**
 * Copyright (C) 2017-2019 Petr Hucik <petr@getdatakick.com>
 *
 * NOTICE OF LICENSE
 *
 * Licensed under the DataKick Regular License version 1.0
 * For more information see LICENSE.txt file
 *
 * @author    Petr Hucik <petr@getdatakick.com>
 * @copyright 2017-2019 Petr Hucik
 * @license   Licensed under the DataKick Regular License version 1.0
 */

namespace Conseqs\Actions;

use Conseqs\Parameters\StringParameterDefinition;
use Conseqs\Action;
use Conseqs\RuntimeModifier;
use Conseqs\ParameterValues;
use Conseqs\ParameterDefinitions;
use PrestaShopException;

class EmailAttachFile extends Action
{
    /**
     * @return string
     */
    public function getName()
    {
        return $this->l('Email: attach file');
    }

    /**
     * @return string[]
     */
    public function getAllowedTriggers()
    {
        return [
            'beforeEmailSent'
        ];
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->l('Attaches custom file to email body');
    }

    /**
     * @return ParameterDefinitions
     */
    public function getSettingsParameters()
    {
        return ParameterDefinitions::none();
    }

    /**
     * @param ParameterValues $settings
     * @return ParameterDefinitions
     */
    public function getInputParameters(ParameterValues $settings)
    {
        return new ParameterDefinitions([
            'filename' => new StringParameterDefinition($this->l('Path to file')),
        ]);
    }


    /**
     * @param ParameterValues $settings
     * @param ParameterValues $input
     * @param ParameterValues $triggerOutput
     * @param RuntimeModifier $runtimeModifier
     * @throws PrestaShopException
     */
    public function execute(ParameterValues $settings, ParameterValues $input, ParameterValues $triggerOutput, RuntimeModifier $runtimeModifier)
    {
        $file = $this->resolveFilePath($input->getValue('filename'));
        $runtimeModifier->adjustParameter('fileAttachment', function($attachments) use ($file) {
            if (! $attachments) {
                $attachments = [];
            }
            if ($attachments && !is_array($attachments)) {
                $attachments = [ $attachments ];
            }
            $attachments[] = [
                'name' => basename($file),
                'content' => file_get_contents($file),
                'mime' => mime_content_type($file)
            ];
            return $attachments;
        });
    }

    /**
     * Returns path to file to be attached
     *
     * @param string $file
     * @return string
     * @throws PrestaShopException
     */
    protected function resolveFilePath($file)
    {
        if (!is_string($file) || !$file) {
            throw new PrestaShopException("Invalid file parameter");
        }

        $candidates = [
            $file,
            rtrim(_PS_ROOT_DIR_, '/') . '/' . ltrim($file, '/'),
            rtrim(_PS_MODULE_DIR_, '/') . '/conseqs/' . ltrim($file, '/'),
            rtrim(_PS_THEME_DIR_, '/') . '/' . ltrim($file, '/'),
        ];
        foreach ($candidates as $candidate) {
            if (file_exists($candidate) && is_file($candidate)) {
                return $candidate;
            }
        }
        throw new PrestaShopException("Can't attach file '$file' to email: file not exists");
    }

}
