<?php
/**
 * Copyright (C) 2017-2019 Petr Hucik <petr@getdatakick.com>
 *
 * NOTICE OF LICENSE
 *
 * Licensed under the DataKick Regular License version 1.0
 * For more information see LICENSE.txt file
 *
 * @author    Petr Hucik <petr@getdatakick.com>
 * @copyright 2017-2019 Petr Hucik
 * @license   Licensed under the DataKick Regular License version 1.0
 */

namespace Conseqs\Actions;

use Conseqs\Parameters\SelectParameterDefinition;
use Conseqs\Parameters\StringParameterDefinition;
use Conseqs\Utils;
use Conseqs\Action;
use Conseqs\RuntimeModifier;
use Conseqs\ParameterValues;
use Conseqs\ParameterDefinitions;
use PrestaShopException;

class ExecuteWebhook extends Action
{
    // settings
    const METHOD = 'method';
    const PAYLOAD = 'payload';
    const URL = 'webhook.url';

    /**
     * @return string
     */
    public function getName()
    {
        return $this->l('Execute webhook');
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->l('Webhooks allow external services to be notified about events. This action will send a GET or POST request to any url you provide');
    }

    /**
     * @return ParameterDefinitions
     */
    public function getSettingsParameters()
    {
        $payload = new StringParameterDefinition($this->l('Payload'), $this->l('Request body to be sent'));
        $payload->setRequired(false);
        $payload->setUiBuilder('webhookPayload');

        $method = new SelectParameterDefinition($this->l('Method'), [
            'GET' => $this->l('GET request'),
            'POST' => $this->l('POST request'),
            'DELETE' => $this->l('DELETE request'),
            'PATCH' => $this->l('PATCH request')
        ], $this->l("HTTP method"));

        return new ParameterDefinitions([
            static::METHOD => $method,
            static::PAYLOAD => $payload,
        ]);
    }

    /**
     * @param ParameterValues $settings
     * @return ParameterDefinitions
     * @throws PrestaShopException
     */
    public function getInputParameters(ParameterValues $settings)
    {
        $statement = $settings->getValue(static::PAYLOAD);
        $parameters = new ParameterDefinitions([
            static::URL => new StringParameterDefinition($this->l('Webhook URL'))
        ]);
        if ($statement) {
            $placeholders = static::getPlaceholders($statement);
            foreach ($placeholders as $key => $placeholder) {
                $parameters->addParameter($key, new StringParameterDefinition($placeholder));
            }
        }
        return $parameters;
    }


    /**
     * @param ParameterValues $settings
     * @param ParameterValues $input
     * @param ParameterValues $triggerOutput
     * @param RuntimeModifier $runtimeModifier
     * @throws PrestaShopException
     */
    public function execute(ParameterValues $settings, ParameterValues $input, ParameterValues $triggerOutput, RuntimeModifier $runtimeModifier)
    {
        $url = $input->getValue(static::URL);
        $data = $settings->getValue(static::PAYLOAD);
        $method = $settings->getValue(static::METHOD);

        $headers = [];
        if ($data) {
            $placeholders = static::getPlaceholders($data);
            foreach ($placeholders as $key => $placeholder) {
                $data = str_replace($placeholder, addcslashes($input->getValue($key), '"\\'), $data);
            }
            $headers[] = 'Content-Type: application/json';
        }

        $curl = curl_init();
        try {
            curl_setopt($curl, CURLOPT_URL, $url);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);
            if ($data) {
                curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
            }
            curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
            curl_exec($curl);
            if (curl_errno($curl)) {
                $errorMsg = curl_error($curl);
                throw new PrestaShopException("Webhook failed: " . $errorMsg);
            }
        } finally {
            curl_close($curl);
        }
    }

    /**
     * @param string $statement
     *
     * @return array
     */
    private static function getPlaceholders($statement)
    {
        $placeholders = Utils::getPlaceholders($statement);
        $ret = [];
        foreach ($placeholders as $placeholder) {
            $key = 'placeholder.' . Utils::camelize(str_replace('}', '', str_replace('{', '', $placeholder)), '_', false);
            $ret[$key] = $placeholder;
        }
        return $ret;

    }
}