<?php
/**
 * Copyright (C) 2017-2019 Petr Hucik <petr@getdatakick.com>
 *
 * NOTICE OF LICENSE
 *
 * Licensed under the DataKick Regular License version 1.0
 * For more information see LICENSE.txt file
 *
 * @author    Petr Hucik <petr@getdatakick.com>
 * @copyright 2017-2019 Petr Hucik
 * @license   Licensed under the DataKick Regular License version 1.0
 */

namespace Conseqs\Actions;

use Conseqs\Compatibility;
use Conseqs\Parameters\IntParameterDefinition;
use Conseqs\Parameters\SelectParameterDefinition;
use PrestaShopException;
use Conseqs\Action;
use Conseqs\RuntimeModifier;
use Conseqs\ParameterValues;
use Conseqs\ParameterDefinitions;
use Module;
use Hook;

class GenzoKronaAction extends Action
{
    /**
     *
     */
    const GENZO_KRONA_MODULE_NAME = 'genzo_krona';

    /**
     * @var bool
     */
    private $kronaInstalled;

    /**
     * Constructor
     */
    public function __construct()
    {
        $this->kronaInstalled = static::checkKronaInstalled();
        $this->addModuleRequirement(static::GENZO_KRONA_MODULE_NAME);
    }

    /**
     * @return bool
     */
    private static function checkKronaInstalled()
    {
        try {
            return (
                Module::isInstalled(static::GENZO_KRONA_MODULE_NAME) &&
                Module::isEnabled(static::GENZO_KRONA_MODULE_NAME)
            );
        } catch (PrestaShopException $e) {
            return false;
        }
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->l('Trigger Krona Action');
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->l('This action will trigger krona action');
    }

    /**
     * @return Compatibility
     */
    public function getCompatibility()
    {
        if ($this->kronaInstalled) {
            return Compatibility::thirtybeesOnly();
        }
        return Compatibility::incompatible();
    }

    /**
     * @param ParameterValues $settings
     * @return ParameterDefinitions
     * @throws PrestaShopException
     */
    public function getInputParameters(ParameterValues $settings)
    {
        if ($this->kronaInstalled) {
            return new ParameterDefinitions([
                'action' => new SelectParameterDefinition($this->l('Krona action'), $this->getKronaActionSelectValues()),
                'customer' => new IntParameterDefinition($this->l('Customer/Player ID')),
            ]);
        }
        return ParameterDefinitions::none();
    }

    /**
     * @param ParameterValues $settings
     * @param ParameterValues $input
     * @param ParameterValues $triggerOutput
     * @param RuntimeModifier $runtimeModifier
     * @return bool
     * @throws PrestaShopException
     */
    public function execute(ParameterValues $settings, ParameterValues $input, ParameterValues $triggerOutput, RuntimeModifier $runtimeModifier)
    {
        $actionId = (int)$input->getValue('action');
        $customerId = (int)$input->getValue('customer');
        if ($actionId && $customerId) {
            $actions = $this->getKronaActions();
            if (isset($actions[$actionId])) {
                $action = $actions[$actionId];
                $data = [
                    'module_name' => $action['module'],
                    'action_name' => $action['key'],
                    'id_customer' => $customerId
                ];
                Hook::exec('actionExecuteKronaAction', $data);
                return true;
            }
        }
        return false;
    }


    /**
     * @return array
     * @throws PrestaShopException
     */
    private function getKronaActions()
    {
        // load module
        Module::getInstanceByName(static::GENZO_KRONA_MODULE_NAME);
        if (class_exists('KronaModule\Action')) {
            /** @noinspection PhpUndefinedNamespaceInspection */
            /** @noinspection PhpUndefinedClassInspection */
            $actions = KronaModule\Action::getAllActions();
            if ($actions) {
                $ret = [];
                foreach ($actions as $action) {
                    $id = (int)$action['id_action'];
                    $ret[$id] = $action;
                }
                return $ret;
            }
        }
        return [];
    }

    /**
     * @return array
     * @throws PrestaShopException
     */
    private function getKronaActionSelectValues()
    {
        return array_map(function($action) {
            return '[' . $action['module']. '] ' . $action['title'];
        }, $this->getKronaActions());
    }
}